<?php
// Own namespace
namespace CoreFramework\Command;

// Import framework stuff
use CoreFramework\Factory\ObjectFactory;
use CoreFramework\Object\BaseFrameworkSystem;
use CoreFramework\Registry\Registry;
use CoreFramework\Request\Requestable;
use CoreFramework\Response\Responseable;

/**
 * A general (base) command
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class BaseCommand extends BaseFrameworkSystem {
	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Sends a generic HTTP response with header, menu, content and footer
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @param	$suffix				Optional template suffix, e.g. '_form' for forms
	 * @return	void
	 */
	protected function sendGenericGetResponse (Requestable $requestInstance, Responseable $responseInstance, $suffix = '') {
		// This command doesn't handle any POST requests, so only handle get request
		assert(!$requestInstance->isPostRequestMethod());

		// Get the application instance
		$applicationInstance = Registry::getRegistry()->getInstance('app');

		// Prepare a template instance
		$templateInstance = $this->prepareTemplateInstance($applicationInstance);

		// Transfer application data
		$templateInstance->assignApplicationData($applicationInstance);

		// Assign base URL
		$templateInstance->assignConfigVariable('base_url');

		// Load the master template
		$masterTemplate = $applicationInstance->buildMasterTemplateName();

		// Load header template
		$templateInstance->loadCodeTemplate('header');

		// Compile and assign it with a variable
		$templateInstance->compileTemplate();
		$templateInstance->assignTemplateWithVariable('header', 'header');

		// Load footer template
		$templateInstance->loadCodeTemplate('footer');

		// Compile and assign it with a variable
		$templateInstance->compileTemplate();
		$templateInstance->assignTemplateWithVariable('footer', 'footer');

		// Load the content template
		$templateInstance->loadCodeTemplate($this->getResolverInstance()->getCommandName() . $suffix);

		// Assign the content template with the master template as a content ... ;)
		$templateInstance->assignTemplateWithVariable($applicationInstance->getAppShortName() . '_' . $this->getResolverInstance()->getCommandName(), 'main_content');

		// Load the master template
		$templateInstance->loadCodeTemplate($masterTemplate);

		// Set title
		$templateInstance->assignVariable('title', $this->getLanguageInstance()->getMessage('page_' . $applicationInstance->getAppShortName() . '_' . $this->getResolverInstance()->getCommandName() . '_title'));

		// Construct the menu in every command. We could do this in BaseCommand class. But this means
		// *every* command has a navigation system and that is want we don't want.
		$menuInstance = ObjectFactory::createObjectByConfiguredName($applicationInstance->getAppShortName() . '_' . $this->getResolverInstance()->getCommandName() . '_menu_class', array($applicationInstance));

		// Render the menu
		$menuInstance->renderMenu();

		// Transfer it to the template engine instance
		$menuInstance->transferContentToTemplateEngine($templateInstance);

		/*
		 * ... and all variables. This should be merged together in a pattern
		 * to make things easier. A cache mechanism should be added between
		 * these two calls to cache compiled templates.
		 */
		$templateInstance->compileVariables();

		// Get the content back from the template engine and put it in response class
		$templateInstance->transferToResponse($responseInstance);
	}

}
